/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#ifndef __DPOINT_H__
#define __DPOINT_H__

#include "revision.h"

class CDPoint
{
public:
	double x;
	double y;


	static double makeXMLUnits( int a )
	{
		return static_cast<double>(a) / static_cast<double>(PIXELSPERMM);
	}

	static int unmakeXMLUnits( double a )
	{
		return static_cast<int>(a * static_cast<double>(PIXELSPERMM));
	}

	CDPoint( CPoint &a )
	{
		x = makeXMLUnits( a.x );
		y = makeXMLUnits( a.y );
	}

	CDPoint( double ix = 0, double iy = 0)
	{
		x = ix;
		y = iy;
	}

	CDPoint operator+( CDPoint a )
	{
		CDPoint r(*this);
		r.x += a.x;
		r.y += a.y;
		return r;
	}

	CDPoint operator+( CPoint a )
	{
		CDPoint r(*this);
		r.x += a.x;
		r.y += a.y;
		return r;
	}
	CDPoint operator-( CPoint a )
	{
		CDPoint r(*this);
		r.x -= a.x;
		r.y -= a.y;
		return r;
	}
};

typedef std::vector<CDPoint>	pointCollection;

#endif
