/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the bus objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "MainFrm.h"
#include "diag.h"
#include "colour.h"
#include "option.h"
#include "LineUtils.h"


////// The BusSlash Class //////

#define SLASH_SIZE 20

void CDrawBusSlash::Load(CStream &archive)
{
  a = ReadPoint(archive);
  archive >> theDir;
  b=a;
  m_segment=0;
}

const char* CDrawBusSlash::GetXMLTag()
{
	return "BUSSLASH";
}

// Load and save to an XML file
void CDrawBusSlash::SaveXML( CXMLWriter &xml )
{
	xml.addTag(GetXMLTag());
	xml.addAttribute( "pos", CDPoint(a) );
	xml.addAttribute( "direction", theDir );
	xml.closeTag();
}

void CDrawBusSlash::LoadXML( CXMLReader &xml )
{
	xml.getAttribute( "pos", a );
	xml.getAttribute( "direction", theDir );
}



CDrawBusSlash::CDrawBusSlash(CTinyCadDoc *pDesign, int NewDir)
: CDrawingObject( pDesign )
{
	theDir = NewDir;
	a=b=CPoint(0,0);
	m_segment=1;
}

BOOL CDrawBusSlash::IsInside(int left,int right,int top,int bottom)
{
	CLineUtils l(a,b);
	return l.IsInside(left,right,top,bottom);
}

double CDrawBusSlash::DistanceFromPoint( CPoint p )
{
	CLineUtils l(a,b);
	CPoint d;
	return l.DistanceFromPoint(p, d );
}


ObjType CDrawBusSlash::GetType()
{
  return xBusSlash;
}

int CDrawBusSlash::getMenuID() 
{ 
	switch (theDir)
	{
	case 0:
		return IDM_BUSSLASH; 
	default:
		return IDM_BUSBACK;
	}
	
}


CString CDrawBusSlash::GetName() const
{
  return "Bus Entry";
}

void CDrawBusSlash::Move(CPoint p)
{
  Display();
  a=p;
  m_segment=0;
  Display();
}

void CDrawBusSlash::LButtonDown(CPoint p, CPoint)
{
  // New undo level for each placement...
  m_pDesign->BeginNewChangeSet();

  Display();
  a=p;
  b=p;
  Store();
  Display();	// Write to screen
}


void CDrawBusSlash::Display( BOOL erase )
{
	// Invalidate the symbol
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, 2 );
}


void CDrawBusSlash::Paint(Context &dc,paint_options options)
{
  CPoint bl;
  switch (options)
  {
  case draw_selected:
	  dc.SelectPen(PS_SOLID,1,cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,1,cPIN_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID,1,m_pDesign->GetOptions()->GetColour( ucPIN ) );
  }

  

	dc.SetROP2(R2_COPYPEN);

  if (theDir == 0)
	b=CPoint(a.x-SLASH_SIZE,a.y+SLASH_SIZE);
  else
	b=CPoint(a.x-SLASH_SIZE,a.y-SLASH_SIZE);

  dc.MoveTo(b);
  dc.LineTo(a);

}


// Store the slash in the drawing
CDrawingObject* CDrawBusSlash::Store()
{
  CDrawBusSlash *NewObject;

  NewObject = new CDrawBusSlash(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  return NewObject;
}



