/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCAD.h"
#include "LibraryStore.h"



// The constructor
CLibraryStore::CLibraryStore()
{
  // Default the name
  m_name="Library";
}

// The destructor
CLibraryStore::~CLibraryStore()
{
}

// Extract a symbol from this library
CLibraryStoreSymbol *CLibraryStore::Extract(const char *key)
{
  CLibraryStoreSymbol *temp=NULL;
  
  if (Symbols.find(key) != Symbols.end())
  {
	  temp = &(Symbols[key]);
  }

  return temp;
}

// Find a string in the symbol names
void CLibraryStore::Find(const char *theString,CListBox *theListBox)
{
	symbolCollection::iterator it = Symbols.begin();

	while (it != Symbols.end())
	{
		CLibraryStoreSymbol *pSymbol = &(it->second);
		CString name = pSymbol->name;
		CString desc = pSymbol->description;
		name.MakeLower();
		desc.MakeLower();
		if (	name.Find(theString) != -1
			||	desc.Find(theString) != -1)
		{
			int index = theListBox->AddString( pSymbol->name + " - " + pSymbol->description );
			theListBox->SetItemDataPtr(index, pSymbol );
		}
		++ it;
	}
}


// Check to see if the item exists in this library
BOOL CLibraryStore::DoesExist( CLibraryStoreSymbol *pSymbol )
{
	symbolCollection::iterator it = Symbols.begin();

	while (it != Symbols.end())
	{
		if (pSymbol == &(it->second))
		{
			return TRUE;
		}
		++ it;
	}

	return FALSE;
}



// Re-read the library
void CLibraryStore::ReRead()
{
  // Get rid of the hash table
  Symbols.erase( Symbols.begin(), Symbols.end() );

  // Re-read the symbol table in
  Attach(m_name);

  // Inform the document views
  static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();
}
