/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <cderr.h>
#include "TinyCadView.h"
#include "revision.h"
#include "registry.h"
#include "resource.h"
#include "diag.h"
#include "option.h"
#include "TinyCad.h"
#include "Object.h"
#include "DlgGetFindBox.h"
#include "MainFrm.h"
#include "StreamClipboard.h"

// The menu commands!
// NOTE: many menu commands are in-line in main.h



//////// the FILE menu ////////


void CTinyCadView::OnFileDesign()
{
  CDlgDetailsBox theDialog(GetDocument(),this);

  if (theDialog.DoModal() == IDOK)
  {
	GetDocument()->SetModifiedFlag( TRUE );
	GetDocument()->Invalidate();
  }
}


void CTinyCadView::OnFileImport() 
{ 
	GetDocument()->SelectObject(new CDrawEditItem(GetDocument())); 
	GetDocument()->UnSelect(); 
	CDrawBlockImport *pImport = new CDrawBlockImport(GetDocument());
	GetDocument()->SelectObject(pImport); 
	pImport->Import( TRUE );
}


void CTinyCadView::OnFilePageSet()
{
  // Get rid of any drawing tool
  GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

  CDlgPageSizeBox theDialog(this,GetDocument()->GetPageBoundries());
  if (theDialog.DoModal()==IDOK) {
	GetDocument()->SetPageBoundries(theDialog.GetSize());
	Invalidate();
  }
}




////// The Find Menu //////

extern CDlgERCListBox theERCListBox;


// Find a string in this design
void CTinyCadView::OnFindFind()
{
	// Get rid of any drawing tool at this moment
	GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

	// Do the dialog
	CDlgFindBox theDialog(this);

	if (theDialog.DoModal()!=IDOK)
		return;

	// Get the string to be found
	CString FindString = theDialog.GetString();

	// Now convert it to lower case only (as this will be a case insensive search)
	FindString.MakeLower();

	
	// Delete all the errors which are currently in the design
	theERCListBox.Close();
	GetDocument()->DeleteErrors();
	theERCListBox.Open(GetDocument());
	int CurrentError = 0;

	// Now look for this string
	CString theFoundString = "";

	drawingIterator it = GetDocument()->GetDrawingBegin();
	while (it != GetDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		theFoundString = pointer->Find(FindString);
		if (theFoundString!="") {
			char buffer[STRLEN];
			strcpy(buffer,pointer->GetName());
			strcat(buffer,": ");
			strcat(buffer,theFoundString);
			GetDocument()->Add(new CDrawError(GetDocument(),pointer->a,CurrentError++));
			theERCListBox.AddString(buffer);
		}
		
		++ it;
	}

	// Was the string found?
	if (CurrentError == 0) {
		Message(IDS_NOFIND);
		theERCListBox.Close();
	}
}


// Set a tag in the design
void CTinyCadView::OnFindSetTag()
{
	CDlgTagBox theDialog(this);

	if (theDialog.DoModal()!=IDOK)
		return;

	// Get the name of the new tag
	CString NewTagName = theDialog.GetString();

	// Now create a new tag with this name
	CDrawTag *newTag = new CDrawTag(GetDocument(),theTransform.GetOrigin(),NewTagName);

	GetDocument()->Add(newTag);
}



// Jump to or delete a tag in this design
void CTinyCadView::OnFindJumpTag()
{			   
 	CDlgJumpTagBox theDialog(this,GetDocument());

	if (theDialog.DoModal()!=IDOK)
		return;

	// Now jump to that tag
	drawingIterator it = GetDocument()->GetDrawingBegin();
	while (it != GetDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;
		if (pointer->GetType()==xTag && ((CDrawTag *)pointer)->GetTagName()==theDialog.GetString())
		{
			theTransform.SetOrigin(pointer->a);
			Invalidate();
		}

		++ it;
	}
}




//////// The VIEW menu ////////

void CTinyCadView::OnViewZoomIn()
{
  CMenu *pMenu = GetMenu();
  LONG NewZoom=theTransform.Scale(1000)*2;

  if (NewZoom>5000)
	NewZoom=5000;
  NewZoom = NewZoom /10;

  SetZoom(NewZoom);

  // Centre at the mouse position
  SetScrollCentre(MousePosition);
}

void CTinyCadView::OnViewZoomOut()
{
  LONG NewZoom;

  NewZoom=theTransform.Scale(1000)/2;
  if (NewZoom<250)
	NewZoom=250;
  SetZoom(NewZoom/10);
}







//////// The TOOLS menu /////////



// Get an object
void CTinyCadView::OnSelectGet()
{
  CLibraryStoreSymbol *theSymbol = static_cast<CMainFrame*>(AfxGetMainWnd())->GetSelectSymbol();
  if (theSymbol==NULL) 
  {
	return;
  }  

  GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));
  hSYMBOL symbol = GetDocument()->GetOptions()->AddSymbol( theSymbol->GetDesignSymbol(GetDocument()) );
  CDrawMethod *NewMethod = new CDrawMethod(GetDocument(), symbol, theSymbol->orientation );
  
  GetDocument()->SelectObject(NewMethod);
}








//////// The Help Menu ////////

void CTinyCadView::OnAbout()
{
  CDialog about( IDD_ABOUTBOX, this );

  about.DoModal();
}





////// The EDIT Menu //////



 void CTinyCadView::OnEditCut()
{
	if (GetDocument()->GetEdit()->GetType() != xEditItem)
		return;

	((CDrawEditItem *)GetDocument()->GetEdit())->ReleaseSelection();
	OnEditCopy();
	GetDocument()->BeginNewChangeSet();
	GetDocument()->SelectDelete();
	Invalidate();
}


// Paste...
void CTinyCadView::OnEditPaste()
{
	OpenClipboard();
	if (IsClipboardAvailable())
	{
		GetDocument()->BeginNewChangeSet();
		GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

		// Now open the file
		CStreamClipboard stream;

		if (stream.ReadFromClipboard( ClipboardFormat ))
		{
			GetDocument()->SelectObject( NULL );

			if (GetDocument()->Import(stream))
			{
				GetDocument()->PostPaste();
				CDrawBlockImport *pImport = new CDrawBlockImport(GetDocument());
				GetDocument()->SelectObject(pImport); 
				pImport->Import( FALSE );
			}
		}

	}
	else if (::IsClipboardFormatAvailable( CF_ENHMETAFILE ))
	{
		// It is a meta-file, we paste these as a new type of object
		HENHMETAFILE clipboard_data = (HENHMETAFILE)::GetClipboardData( CF_ENHMETAFILE );
		CDrawMetaFile *pObject = new CDrawMetaFile( GetDocument() );
		pObject->setMetaFile( clipboard_data );
		GetDocument()->Add( pObject );
		GetDocument()->UnSelect();
		CDrawBlockImport *pImport = new CDrawBlockImport(GetDocument());
		GetDocument()->SelectObject(pImport); 
		GetDocument()->Select( pObject );
		pImport->Import( FALSE );
	}
	else
	{
		AfxMessageBox("Cannot paste");
	}

	CloseClipboard();
}

 void CTinyCadView::OnDestroyClipboard()
{
	if (IsClipboardAvailable()) 
	{
		char theName[MAX_PATH],TempPath[MAX_PATH];
		GetTempPath(sizeof(TempPath),TempPath);
		GetTempFileName(TempPath,"$DS",42,theName);

		::DeleteFile(theName);
	}

	CWnd::OnDestroyClipboard();	
}

BOOL CTinyCadView::IsClipboardAvailable()
{
	return ::IsClipboardFormatAvailable(ClipboardFormat);
}

void CTinyCadView::OnEditDuplicate()
{
	if (GetDocument()->GetEdit()->GetType() != xEditItem)
		return;
	OnEditCopy();
	OnEditPaste();
}

void CTinyCadView::OnEditCopy()
{
	if (GetDocument()->GetEdit()->GetType() != xEditItem)
		return;

	OpenClipboard();
	EmptyClipboard();

	CClientDC ref_dc(this);
	HENHMETAFILE hmeta = GetDocument()->CreateMetafile( ref_dc );

	if (hmeta)
	{
		SetClipboardData(CF_ENHMETAFILE,hmeta);
	}

	// Save the selected area
	CStreamClipboard stream;
	CXMLWriter xml( &stream );
	GetDocument()->SaveXML(xml,TRUE,TRUE);
	stream.SaveToClipboard(ClipboardFormat);

	CloseClipboard();
}


void CTinyCadView::OnEditCopyto() 
{
	if (GetDocument()->IsSelected())
	{
		GetDocument()->Save(TRUE,TRUE);
	}
}



void CTinyCadView::OnSelectArc() 
{ 
	g_EditToolBar.m_LineEdit.SetArcAngle();
	GetDocument()->SelectObject(new CDrawPolygon(GetDocument())); 
}

void CTinyCadView::OnSelectPolygon() 
{ 
	g_EditToolBar.m_LineEdit.SetLineAngle();
	GetDocument()->SelectObject(new CDrawPolygon(GetDocument())); 
}


void CTinyCadView::OnSelectJunction()
{ 
	if (!GetDocument()->GetOptions()->GetAutoJunc())
	{
		GetDocument()->SelectObject(new CDrawJunction(GetDocument())); 
	}
	else
	{
		AfxMessageBox( IDS_NOJUNCTION );
	}
}

