/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __REGISTRY_H__
#define __REGISTRY_H__

#include <atlbase.h>
#include "revision.h"


//*************************************************************************
//* interface to the registry database                                    *
//*                                                                       *
//* Each registry entry is placed into                                    *
//* HKEY_CURRENT_USER\Software\TinyCAD\TinyCAD                            *
//* (where name is specified on construction of registry)                 *
//* They are then sub divided into keys.                                  *
//* Each key is a single string in the database                           *
//*                                                                       *
//*************************************************************************
class Registry 
{
	//=====================================================================
	//== constructor/destructor/initializing                             ==
	//=====================================================================
	public:					Registry();
	public:					~Registry();

	//=====================================================================
	//== accessor/mutator                                                ==
	//=====================================================================
	public:static void		Set( CString sKey, DWORD nVal );					
	public:static void		Set( CString sKey, WORD nVal );					
	public:static void		Set( CString sKey, int nVal );					
	public:static void		Set( CString sKey, bool bVal );					
	public:static void		Set( CString sKey, CSize szVal );					
	public:static void		Set( CString sKey, CString sVal );			
	private:static void		Set( CString sKey, void* data,int length );	

	public:static CString	GetString(	CString sKey, CString sDefaultVal );
	public:static bool		GetBool(	CString sKey, bool bDefaultVal );
	public:static BOOL		GetBOOL(	CString sKey, BOOL nDefaultVal );
	public:static int		GetInt(		CString sKey, int nDefaultVal );
	public:static char		GetChar(	CString sKey, char nDefaultVal );
	public:static WORD		GetWord(	CString sKey, WORD nDefaultVal );
	public:static LONG		GetLong(	CString sKey, LONG nDefaultVal );
	public:static CSize		GetSize(	CString sKey, CSize szDefault );
	private:static bool		GetRaw(		CString sKey, char* data, unsigned int length);	
	public:static CStringList* GetStringList( CString sKey );


	//=====================================================================
	//== These are some standard functions which use the registry        ==
	//=====================================================================
	public:static void		SetPrinterMode(HANDLE theDevMode,HANDLE thePrinterName);

	//-- Associate this software with some documents
	private:static	void	Associate( CString sExtension, CString sTitle, CString sProgram );
	
	//-- Check key
	private:static  bool	keyExists( CString sKey );

	//-- Write the initial data to the registry
	private:static  void	CreateDefaultEntries();

	private:static	int		QueryValueEx(CString sKey,DWORD *Type,unsigned char* data,unsigned long *size);
	private:static  int		SetValueEx(CString sKey,DWORD Type, unsigned char* data, unsigned long length);

	//=====================================================================
	//== class constants                                                 ==
	//=====================================================================
	// The revision number for storage in the registry database
	private: static const CString	M_SKEY; 

	//=====================================================================
	//== class variables                                                 ==
	//=====================================================================
	private: static CRegKey			m_oKey;
	
	// Forces a ctor call before calling any static method
	// Simulates a java static ctor block
	private: static Registry		m_oDummi;
};
//=========================================================================


//*************************************************************************
//** The Find File class                                                 **
//*************************************************************************
class FindFile 
{
		HANDLE	handle;
		WIN32_FIND_DATA fileinfo;

public:
	FindFile(CString);
	virtual ~FindFile();
	bool 	FindNext();
	char 	*GetName();
	bool	Success();
};
//=========================================================================
#endif

