/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "TinyCadDoc.h"
#include "TinyCadView.h"
#include "Registry.h"
#include "LibraryView.h"
#include "LibraryDoc.h"
#include "TinyCadSymbolDoc.h"
#include "LibraryDb.h"
#include "DlgLibraryBox.h"
#include "Object.h"
#include "LibraryCollection.h"
//#include "AboutDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp

BEGIN_MESSAGE_MAP(CTinyCadApp, CWinApp)
	//{{AFX_MSG_MAP(CTinyCadApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(IDM_LIBLIB, OnLiblib)
	ON_COMMAND(ID_HELP_GOTOTINYCADWEBSITE, OnHelpGototinycadwebsite)
	ON_COMMAND(ID_HELP_HELP, OnHelpHelp)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp construction

CTinyCadApp::CTinyCadApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTinyCadApp object

CTinyCadApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp initialization

BOOL CTinyCadApp::InitInstance()
{
	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();


	//
	// Force system to use DA0 3.60 not 3.50.
	//
	AfxGetModuleState()->m_dwVersion = 0x0601;


	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey( "TinyCAD" );

	LoadStdProfileSettings(6);  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	m_pDocTemplate = new CMultiDocTemplate(
		IDR_TCADTYPE,
		RUNTIME_CLASS(CTinyCadDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTinyCadView));
	AddDocTemplate(m_pDocTemplate);

	m_pLibTemplate = new CMultiDocTemplate(
		IDR_LIBTYPE,
		RUNTIME_CLASS(CLibraryDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CLibraryView));
	AddDocTemplate(m_pLibTemplate);

	// Read the registry
	RegistryRead();

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);


	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	if (m_Maximize && m_nCmdShow == 1)
	{
		m_nCmdShow = SW_SHOWMAXIMIZED;
	}

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString	m_Version;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_Version = _T("");
	//}}AFX_DATA_INIT
	m_Version = "TinyCAD " + static_cast<CTinyCadApp*>(AfxGetApp())->get_version();
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_VERSION, m_Version);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CTinyCadApp::OnAppAbout()
{
	//DAboutDlg().DoModal();
	CAboutDlg().DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CTinyCadApp message handlers

//-------------------------------------------------------------------------
// This is the idle time processing
BOOL CTinyCadApp::OnIdle(LONG count)
{
  CWinApp::OnIdle(count);

  	switch (count)
  	{
	case 0:
			CLibraryCollection::DoIdle();

	default:
		break;
  }

  return FALSE;
}
//-------------------------------------------------------------------------
int CTinyCadApp::ExitInstance() 
{
	// Delete the libraries
	CLibraryCollection::Clear();
	
	return CWinApp::ExitInstance();
}
//-------------------------------------------------------------------------
// Read the registry
void CTinyCadApp::RegistryRead()
{
	// Is there a list of libraries in the registry?
	CStringList*	colLibs = Registry::GetStringList( "Libraries" );

	// One single library
	CString			sLibName;

	CLibraryStore*	oLib;


	// Iterate through the list in head-to-tail order.
	for( POSITION pos = colLibs->GetHeadPosition(); pos != NULL; )
	{
		sLibName = colLibs->GetNext( pos );

		// Is this a new file library or an old library type?
		FILE* f = fopen( sLibName + ".idx", "rb" );

		if( f )
		{
			fclose(f);
			oLib = new CLibraryFile;
		}
		else
		{
			oLib = new CLibraryDb;
		}
		
		oLib->Attach( sLibName );
		CLibraryCollection::Add( oLib );
	}

	delete colLibs;

	m_Maximize		= Registry::GetBool( "Maximize",		false );
	m_MdiMaximize	= Registry::GetBool( "MdiMaximize",		false );
	m_ShowPosition	= Registry::GetBool( "ShowPosition",	false );
	m_PrintBandW	= Registry::GetBool( "PrintBandW",		false );
	m_PrintScale	= Registry::GetInt(	 "PrintScale",		100 );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetMaximize( bool bHasMaxScreen )
{
	m_Maximize = bHasMaxScreen;
	Registry::Set( "Maximize", m_Maximize );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetMdiMaximize( bool bHasMaxMdi )
{
	m_MdiMaximize = bHasMaxMdi;
	Registry::Set( "MdiMaximize", m_MdiMaximize );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetPrintBandW( bool m )
{
	m_PrintBandW = m;
	Registry::Set( "PrintBandW", m_PrintBandW );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetPrintScale( int m )
{
	m_PrintScale = m;
	Registry::Set( "PrintScale", m_PrintScale );
}
//-------------------------------------------------------------------------
// Edit a library using the doc/view
void CTinyCadApp::editLibrary(CLibraryStore *library)
{
	// Is this library already being edited?
	POSITION p;
	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( library ))
		{
			POSITION v = t->GetFirstViewPosition();
			
			// Active it's views
		   while (v != NULL)
		   {
			  CView* pView = t->GetNextView(v);
			  pView->GetParentFrame()->BringWindowToTop();
		   }   

		   return;
		}		
	}

	
	CLibraryDoc *pDoc = static_cast<CLibraryDoc *>(m_pLibTemplate->CreateNewDocument());

	if (pDoc != NULL)
	{
		pDoc->setLibrary( library );
		CFrameWnd *pFrame = m_pLibTemplate->CreateNewFrame( pDoc, NULL );
		m_pLibTemplate->InitialUpdateFrame( pFrame, pDoc, TRUE );
	}
}

////// the Library Menu //////

void CTinyCadApp::OnLiblib() 
{
  CDlgLibraryBox theDialog(AfxGetMainWnd());

  theDialog.DoModal();
}


void CTinyCadApp::ResetAllSymbols()
{
	static_cast<CMainFrame*>(AfxGetMainWnd())->ResetAllSymbols();
	
	POSITION p;
	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		t->ResetSymbols();
	}
}




bool CTinyCadApp::IsLibInUse( CLibraryStore *lib )
{

	POSITION p;
	p = m_pDocTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CTinyCadDoc *t = static_cast<CTinyCadDoc*>(m_pDocTemplate->GetNextDoc(p));
		if (t->IsLibInUse( lib ))
		{
			return true;
		}
	}

	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( lib ))
		{
			return true;
		}		
	}

	return false;
}

// Edit a library using the doc/view
void CTinyCadApp::editSymbol(CLibraryStore *library, CLibraryStoreSymbol &symbol)
{
	CTinyCadSymbolDoc *pDoc = new CTinyCadSymbolDoc();
	pDoc->setSymbol( library, symbol );

	CFrameWnd *pFrame = m_pDocTemplate->CreateNewFrame( pDoc, NULL );
	m_pDocTemplate->InitialUpdateFrame( pFrame, pDoc, TRUE );
}

void CTinyCadApp::OnHelpGototinycadwebsite() 
{
	// Open a browser for our web site
	ShellExecute( AfxGetMainWnd()->m_hWnd, "open", "http://tinycad.sourceforge.net", NULL, NULL, SW_SHOWNORMAL );	
}




CString CTinyCadApp::get_version()
{
	char     szModulePath[MAX_PATH];
	DWORD    dwSize;
	DWORD    dwZero;
	char*    pBuffer;
	VS_FIXEDFILEINFO* pFixedInfo;
	UINT    uVersionLen;
 
	CString r;

	GetModuleFileName(NULL, szModulePath, MAX_PATH);
	dwSize = GetFileVersionInfoSize(szModulePath, &dwZero);
 
	if (dwSize > 0)
	{
		pBuffer = (char *)malloc(dwSize * sizeof(char));
 
		if( !pBuffer )
		{
			return r;
		}

		GetFileVersionInfo(szModulePath, dwZero, dwSize, pBuffer);
		VerQueryValue(pBuffer, _T("\\"), (void**) &pFixedInfo, (UINT*)
		&uVersionLen);

		r.Format( "version %u.%02u.%02u", HIWORD(pFixedInfo->dwProductVersionMS),
									LOWORD(pFixedInfo->dwProductVersionMS),
									HIWORD(pFixedInfo->dwProductVersionLS));
 
		free( pBuffer );
	}

	return r;
}



bool CTinyCadApp::IsWinNT()
{
	static bool checked = false;
	static bool r = true;

	if (!checked)
	{
		checked = true;

		OSVERSIONINFO version;
		version.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
		GetVersionEx( &version );
	
		r = version.dwPlatformId == VER_PLATFORM_WIN32_NT;
	}

	return r;
}
