/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCad.h"

#include "SpecialPrintDlg.h"
#include "TinyCadDoc.h"
#include "TinyCadView.h"

#include "OptionsPropertySheet.h"
#include "DlgExportPng.h"
#include "DlgColours.h"


#include "MainFrm.h"

#include "registry.h"
#include "diag.h"

#include "EditToolbar.h"
#include "DlgPositionBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




/////////////////////////////////////////////////////////////////////////////
// CTinyCadView

IMPLEMENT_DYNCREATE(CTinyCadView, CView)

BEGIN_MESSAGE_MAP(CTinyCadView, CView)
	//{{AFX_MSG_MAP(CTinyCadView)
	ON_UPDATE_COMMAND_UI(IDM_EDITEDIT, OnUpdateEditedit)
	ON_UPDATE_COMMAND_UI(IDM_BUSBACK, OnUpdateBusback)
	ON_UPDATE_COMMAND_UI(IDM_BUSSLASH, OnUpdateBusslash)
	ON_UPDATE_COMMAND_UI(IDM_TOOLARC, OnUpdateToolarc)
	ON_UPDATE_COMMAND_UI(IDM_TOOLBUS, OnUpdateToolbus)
	ON_UPDATE_COMMAND_UI(IDM_TOOLBUSNAME, OnUpdateToolbusname)
	ON_UPDATE_COMMAND_UI(IDM_TOOLCIRCLE, OnUpdateToolcircle)
	ON_UPDATE_COMMAND_UI(IDM_TOOLCONNECT, OnUpdateToolconnect)
	ON_UPDATE_COMMAND_UI(IDM_TOOLGET, OnUpdateToolget)
	ON_UPDATE_COMMAND_UI(IDM_TOOLJUNC, OnUpdateTooljunc)
	ON_UPDATE_COMMAND_UI(IDM_TOOLLABEL, OnUpdateToollabel)
	ON_UPDATE_COMMAND_UI(IDM_TOOLPOLYGON, OnUpdateToolpolygon)
	ON_UPDATE_COMMAND_UI(IDM_TOOLPOWER, OnUpdateToolpower)
	ON_UPDATE_COMMAND_UI(IDM_TOOLSQUARE, OnUpdateToolsquare)
	ON_UPDATE_COMMAND_UI(IDM_TOOLTEXT, OnUpdateTooltext)
	ON_UPDATE_COMMAND_UI(IDM_TOOLWIRE, OnUpdateToolwire)
	ON_UPDATE_COMMAND_UI(IDM_VIEWCENTRE, OnUpdateViewcentre)
	ON_WM_LBUTTONUP()
	ON_WM_SETCURSOR()
	ON_WM_SIZE()
	ON_UPDATE_COMMAND_UI(IDM_EDITDRAG, OnUpdateEditdrag)
	ON_UPDATE_COMMAND_UI(IDM_EDITDUP, OnUpdateEditdup)
	ON_UPDATE_COMMAND_UI(IDM_EDITROTATE, OnUpdateEditrotate)
	ON_UPDATE_COMMAND_UI(IDM_EDITMOVE, OnUpdateEditmove)
	ON_COMMAND(IDM_SNAPTOGRID, OnSnaptogrid)
	ON_UPDATE_COMMAND_UI(IDM_SNAPTOGRID, OnUpdateSnaptogrid)
	ON_UPDATE_COMMAND_UI(IDM_REPEATNAMEDOWN, OnUpdateRepeatnamedown)
	ON_UPDATE_COMMAND_UI(IDM_REPEATNAMEUP, OnUpdateRepeatnameup)
	ON_UPDATE_COMMAND_UI(IDM_REPEATPINDOWN, OnUpdateRepeatpindown)
	ON_UPDATE_COMMAND_UI(IDM_REPEATPINUP, OnUpdateRepeatpinup)
	ON_UPDATE_COMMAND_UI(IDM_EDITPASTE, OnUpdateEditpaste)
	ON_UPDATE_COMMAND_UI(IDM_EDITCUT, OnUpdateEditcut)
	ON_UPDATE_COMMAND_UI(IDM_EDITCOPY, OnUpdateEditcopy)
	ON_WM_DESTROY()
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_LIBRARY_REFRESHSYMBOLSFROMLIBRARIES, OnLibraryRefreshsymbolsfromlibraries)
	ON_COMMAND(ID_RULER_VERT, OnRulerVert)
	ON_UPDATE_COMMAND_UI(ID_RULER_VERT, OnUpdateRulerVert)
	ON_COMMAND(ID_RULER_HORIZ, OnRulerHoriz)
	ON_UPDATE_COMMAND_UI(ID_RULER_HORIZ, OnUpdateRulerHoriz)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(IDM_VIEW_OPTIONS, OnViewOptions)
	ON_COMMAND(ID_CONTEXT_ARCIN, OnContextArcin)
	ON_COMMAND(ID_CONTEXT_ARCOUT, OnContextArcout)
	ON_COMMAND(ID_CONTEXT_CANCELDRAWING, OnContextCanceldrawing)
	ON_COMMAND(ID_CONTEXT_FINISHDRAWING, OnContextFinishdrawing)
	ON_COMMAND(ID_CONTEXT_FREELINE, OnContextFreeline)
	ON_COMMAND(ID_CONTEXT_ADDHANDLE, OnContextAddhandle)
	ON_COMMAND(ID_CONTEXT_DELETEHANDLE, OnContextDeletehandle)
	ON_COMMAND(ID_CONTEXT_ZORDER_BRINGTOFRONT, OnContextZorderBringtofront)
	ON_COMMAND(ID_CONTEXT_ZORDER_SENDTOBACK, OnContextZorderSendtoback)
	ON_UPDATE_COMMAND_UI(ID_CONTEXT_ZORDER_BRINGTOFRONT, OnUpdateContextZorderBringtofront)
	ON_UPDATE_COMMAND_UI(ID_CONTEXT_ZORDER_SENDTOBACK, OnUpdateContextZorderSendtoback)
	ON_UPDATE_COMMAND_UI(IDM_EDITDUPLICATE, OnUpdateEditduplicate)
	ON_COMMAND(ID_SPECIAL_CREATESPICEFILE, OnSpecialCreatespicefile)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPYTO, OnUpdateEditCopyto)
	ON_COMMAND(ID_EDIT_COPYTO, OnEditCopyto)
	ON_COMMAND(ID_CONTEXT_MAKEHORIZONTAL, OnContextMakehorizontal)
	ON_COMMAND(ID_CONTEXT_MAKEVERTICAL, OnContextMakevertical)
	ON_COMMAND(ID_FILE_SAVEASBITMAP, OnFileSaveasbitmap)
	ON_COMMAND(ID_OPTIONS_COLOURS, OnOptionsColours)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)

	ON_WM_INITMENUPOPUP()
	ON_WM_COMPACTING()
	ON_WM_DESTROYCLIPBOARD()
	ON_WM_CREATE()
	ON_WM_DESTROY()

	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_MBUTTONDOWN()
	ON_WM_MBUTTONUP()
	ON_WM_MOUSEMOVE()

	ON_WM_CLOSE()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_SIZE()

	// The File Menu
	ON_COMMAND( IDM_FILEDESIGN, OnFileDesign )
	ON_COMMAND( IDM_FILEIMPORT, OnFileImport )
	ON_COMMAND( ID_FILE_PRINT, OnFilePrint )
	ON_COMMAND( IDM_FILEPAGESET, OnFilePageSet )

	// The Edit Menu
	ON_COMMAND( IDM_EDITUNDO, OnEditUndo )
	ON_COMMAND( IDM_EDITREDO, OnEditRedo )
	ON_COMMAND( IDM_EDITEDIT, OnEditEdit )
	ON_COMMAND( IDM_EDITDELITEM, OnEditDelete )
	ON_COMMAND( IDM_EDITDRAG, OnEditDrag )
	ON_COMMAND( IDM_EDITMOVE, OnEditMove )
	ON_COMMAND( IDM_EDITDUP, OnEditDup )
	ON_COMMAND( IDM_EDITROTATE, OnEditRotate )
	ON_COMMAND( ID_FIND_FIND, OnFindFind )
	ON_COMMAND( ID_FIND_SETTAG, OnFindSetTag )
	ON_COMMAND( ID_FIND_TAG, OnFindJumpTag )
	ON_COMMAND( IDM_EDITCOPY, OnEditCopy )
	ON_COMMAND( IDM_EDITCUT, OnEditCut )
	ON_COMMAND( IDM_EDITPASTE, OnEditPaste )
	ON_COMMAND( IDM_EDITDUPLICATE, OnEditDuplicate )


	// The Keyboard options
	ON_COMMAND( IDM_VIEWZOOMIN, OnViewZoomIn )
	ON_COMMAND( IDM_VIEWZOOMOUT, OnViewZoomOut )
	
	// The Toolbar Menu
	ON_COMMAND( IDM_VIEWCENTRE, OnViewCentre )
	ON_COMMAND( IDM_TOOLBUSNAME, OnSelectBusName )
	ON_COMMAND( IDM_TOOLBUS, OnSelectBus )
	ON_COMMAND( IDM_BUSSLASH, OnSelectBusSlash )
	ON_COMMAND( IDM_BUSBACK, OnSelectBusBack )
	ON_COMMAND( IDM_TOOLCONNECT, OnSelectConnect )
	ON_COMMAND( IDM_TOOLLABEL, OnSelectLabel )
	ON_COMMAND( IDM_TOOLPOWER, OnSelectPower )
	ON_COMMAND( IDM_TOOLPOLYGON, OnSelectPolygon )
	ON_COMMAND( IDM_TOOLWIRE, OnSelectWire )
	ON_COMMAND( IDM_TOOLJUNC, OnSelectJunction )
	ON_COMMAND( IDM_TOOLARC, OnSelectArc )
	ON_COMMAND( IDM_TOOLSQUARE, OnSelectSquare )
	ON_COMMAND( IDM_TOOLCIRCLE, OnSelectCircle )
	ON_COMMAND( IDM_TOOLTEXT, OnSelectText )
	ON_COMMAND( IDM_TOOLGET, OnSelectGet )

	// The Special Menu
	ON_COMMAND( IDM_SPECIALANOTATE, OnSpecialAnotate )
	ON_COMMAND( IDM_SPECIALBOM, OnSpecialBom )
	ON_COMMAND( IDM_SPECIALNET, OnSpecialNet )
	ON_COMMAND( IDM_SPECIALCHECK, OnSpecialCheck )

	// The Repeat Menu
	ON_COMMAND( IDM_REPEATNAMEUP, OnRepeatNameUp )
	ON_COMMAND( IDM_REPEATNAMEDOWN, OnRepeatNameDown )
	ON_COMMAND( IDM_REPEATPINUP, OnRepeatPinUp )
	ON_COMMAND( IDM_REPEATPINDOWN, OnRepeatPinDown )

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTinyCadView construction/destruction

CTinyCadView::CTinyCadView()
{
  vRuler = NULL;
  hRuler = NULL;

  // We have not captured the mouse
  m_captured = FALSE;
  m_panning = FALSE;

}

CTinyCadView::~CTinyCadView()
{
}

BOOL CTinyCadView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// CTinyCadView printing

// Our own version of this function, so that we
// can set portrait/landscape mode automatically
BOOL CTinyCadView::DoPreparePrinting(CPrintInfo* pInfo)
{
	ASSERT(pInfo != NULL);
	ASSERT(pInfo->m_pPD != NULL);

	if (pInfo->m_pPD->m_pd.nMinPage > pInfo->m_pPD->m_pd.nMaxPage)
		pInfo->m_pPD->m_pd.nMaxPage = pInfo->m_pPD->m_pd.nMinPage;

	// don't prompt the user if we're doing print preview, printing directly,
	// or printing via IPrint and have been instructed not to ask

	CTinyCadApp* pApp = static_cast<CTinyCadApp*>(AfxGetApp());
	if (pInfo->m_bPreview || pInfo->m_bDirect ||
		(pInfo->m_bDocObject && !(pInfo->m_dwFlags & PRINTFLAG_PROMPTUSER)))
	{
		if (pInfo->m_pPD->m_pd.hDC == NULL)
		{
			// if no printer set then, get default printer DC and create DC without calling
			//   print dialog.
			if (!pApp->GetPrinterDeviceDefaults(&pInfo->m_pPD->m_pd))
			{
				// bring up dialog to alert the user they need to install a printer.
				if (!pInfo->m_bDocObject || (pInfo->m_dwFlags & PRINTFLAG_MAYBOTHERUSER))
					if (pApp->DoPrintDialog(pInfo->m_pPD) != IDOK)
						return FALSE;
			}

			if (pInfo->m_pPD->m_pd.hDC == NULL)
			{
				// Switch to landscape mode...
				DEVMODE *pdev_mode;
				pdev_mode = pInfo->m_pPD->GetDevMode();
				pdev_mode->dmOrientation = GetDocument()->IsPortrait() ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
				GlobalUnlock( pdev_mode );
				
				// call CreatePrinterDC if DC was not created by above
				if (pInfo->m_pPD->CreatePrinterDC() == NULL)
					return FALSE;
			}
		}

		// set up From and To page range from Min and Max
		pInfo->m_pPD->m_pd.nFromPage = (WORD)pInfo->GetMinPage();
		pInfo->m_pPD->m_pd.nToPage = (WORD)pInfo->GetMaxPage();
	}
	else
	{
		// Get the print scaling to determine the correct "fit to page"
		// scaling factor
		CPrintDialog pdlg( FALSE, 0, AfxGetMainWnd() );
		HDC hdc = NULL;

		if (AfxGetApp()->GetPrinterDeviceDefaults(&pdlg.m_pd))
		{
			if (pdlg.m_pd.hDC == NULL)
			{
				// call CreatePrinterDC if DC was not created by above
				hdc = pdlg.CreatePrinterDC();
			}
		}
		
		int width = 1024;
		int height = 768;
		if (hdc) 
		{
			// Change the scaling to print correct size
			int x=::GetDeviceCaps(hdc,HORZSIZE);
			int y=::GetDeviceCaps(hdc,VERTSIZE);
			width=max(x,y);
			height=min(x,y);
		}

		::DeleteDC( hdc );

	
		// otherwise, bring up the print dialog and allow user to change things
		// preset From-To range same as Min-Max range
		delete pInfo->m_pPD;
		CSpecialPrintDlg *dlg = new CSpecialPrintDlg( FALSE );
		dlg->m_FitScale = 
			(width * 100 * PIXELSPERMM) / GetDocument()->GetPageBoundries().x;
		pInfo->m_pPD = dlg;
		pInfo->m_pPD->m_pd.nFromPage = (WORD)pInfo->GetMinPage();
		pInfo->m_pPD->m_pd.nToPage = (WORD)pInfo->GetMaxPage();
		pInfo->m_pPD->m_pd.Flags &= ~PD_RETURNDC;

		if (pApp->DoPrintDialog(pInfo->m_pPD) != IDOK)
			return FALSE;       // do not print

		// Switch to landscape mode...
		DEVMODE *pdev_mode;
		pdev_mode = dlg->GetDevMode();
		pdev_mode->dmOrientation = GetDocument()->IsPortrait() ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
		pdev_mode->dmCopies = dlg->m_Copies;
		GlobalUnlock( pdev_mode );

		// call CreatePrinterDC if DC was not created by above
		if (pInfo->m_pPD->CreatePrinterDC() == NULL)
			return FALSE;

	}

	ASSERT(pInfo->m_pPD != NULL);
	ASSERT(pInfo->m_pPD->m_pd.hDC != NULL);
	if (pInfo->m_pPD->m_pd.hDC == NULL)
		return FALSE;

	pInfo->m_nNumPreviewPages = pApp->m_nNumPreviewPages;
	VERIFY(pInfo->m_strPageDesc.LoadString(AFX_IDS_PREVIEWPAGEDESC));
	return TRUE;
}



BOOL CTinyCadView::OnPreparePrinting(CPrintInfo* pInfo)
{

	// Get rid of any drawing tool
	GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

	// Prepare using our version of this function to force
	// portrait/landscape mode
	return DoPreparePrinting(pInfo);
}

void CTinyCadView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// Calculate how many cols and rows of pages to use
	double width = pDC->GetDeviceCaps(HORZSIZE) * PIXELSPERMM;
	double height = pDC->GetDeviceCaps(VERTSIZE) * PIXELSPERMM;

	double scale_x = static_cast<double>(GetDocument()->GetPageBoundries().x) / width;
	double scale_y = static_cast<double>(GetDocument()->GetPageBoundries().y) / height;

	m_Fit = FALSE;

	if (m_Fit) 
	{
		m_cols = 1;
		m_rows = 1;
	} 
	else 
	{
		// Round to the nearest scaling factor...
		m_cols = scale_x;
		m_rows = scale_y;

		double sx = scale_x - static_cast<int>(scale_x);
		double sy = scale_y - static_cast<int>(scale_y);

		// Is this an exact fit or not?
		if (sx > 0.01)
		{
			m_cols ++;
		}

		if (sy > 0.01)
		{
			m_rows ++;
		}
	}
	
	pInfo->SetMinPage( 1 );
	pInfo->SetMaxPage(m_cols * m_rows );
	
	// Change the zoom so a single unit on the design corresponds to a pixel on the printer
	current_Transform = theTransform;
	double NewZoom = (pDC->GetDeviceCaps(LOGPIXELSX)*1000) / (PIXELSPERMM*254);

	if (m_Fit)
	{
		NewZoom = NewZoom / max(scale_x,scale_y);
	}
	else
	{
		int scale = ((CTinyCadApp*)AfxGetApp())->GetPrintScale();
		NewZoom = (NewZoom * scale) / 100;
	}

	theTransform.SetZoom(NewZoom);
}

void CTinyCadView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{

	if (pInfo)
	{
		// Change the offset co-ordinates to print the correct portion of the design
		int width = pDC->GetDeviceCaps(HORZSIZE) * PIXELSPERMM;
		int height = pDC->GetDeviceCaps(VERTSIZE) * PIXELSPERMM;
		int col = (pInfo->m_nCurPage-1) / m_rows;
		int row = (pInfo->m_nCurPage-1) % m_rows;
		int x=col * width;
		int y=row * height;
		theTransform.SetOriginX(x);
		theTransform.SetOriginY(y);
	}
	
	CView::OnPrepareDC(pDC, pInfo);
}


void CTinyCadView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	theTransform = current_Transform;
}

/////////////////////////////////////////////////////////////////////////////
// CTinyCadView diagnostics

#ifdef _DEBUG
void CTinyCadView::AssertValid() const
{
	CView::AssertValid();
}

void CTinyCadView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTinyCadDoc* CTinyCadView::GetDocument() // non-debug version is inline
{
	return (CTinyCadDoc*)m_pDocument;
}
#endif //_DEBUG




/////////////////////////////////////////////////////////////////////////////
// CTinyCadView message handlers

class CDlgPositionBox;
// extern CDlgPositionBox thePositionBox;
#define thisDesign (&((static_cast<CTinyCadView *>(m_pMainWnd))->theDesign))


/////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////



// The status bar indicators
static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,			// status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
};



// The OnCreate function, called when the window is created
int CTinyCadView::OnCreate( LPCREATESTRUCT q )
{
	CView::OnCreate( q );

	// Now create the new rulers
  	CRect nSize;
  	GetClientRect(nSize);
  	vRuler = new Ruler(GetDocument(), 0, nSize, theTransform,this);
  	hRuler = new Ruler(GetDocument(), 2, nSize, theTransform,this);

  // No zoom (100% size)
  SetZoom(100);


  // Set the offsets
  SetScroll(0,0);

  ClipboardFormat = RegisterClipboardFormat(CLIPBOARD_FORMAT);


	// initialize the cursor array
	HINSTANCE hInst = AfxGetResourceHandle();
	m_mouse_pointers[0] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACKNWSE));
	m_mouse_pointers[1] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACKNESW));
	m_mouse_pointers[2] = m_mouse_pointers[0];
	m_mouse_pointers[3] = m_mouse_pointers[1];
	m_mouse_pointers[4] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACKNS));
	m_mouse_pointers[5] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACKWE));
	m_mouse_pointers[6] = m_mouse_pointers[4];
	m_mouse_pointers[7] = m_mouse_pointers[5];
	m_mouse_pointers[8] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACK_MOVE));
	m_mouse_pointers[9] = ::LoadCursor(hInst, MAKEINTRESOURCE(IDC_ZOOMCURSOR));
	m_mouse_pointers[10] = ::LoadCursor(hInst, MAKEINTRESOURCE(IDC_REFCURSOR));
	m_mouse_pointers[11] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACK4WAY));
	m_mouse_pointers[12] = ::LoadCursor(hInst, MAKEINTRESOURCE(AFX_IDC_TRACK_BLOCK));


	return 0;
}


// CTinyCadView constructor:
// Create the window with the appropriate style, size, menu, etc.
//


// Called when the window is about to be destroyed
void CTinyCadView::OnDestroy()
{


	vRuler->DestroyWindow();
	delete vRuler;

	hRuler->DestroyWindow();
	delete hRuler;
}




// Display a standard message using one of the resource strings
int Message(int Resource,int Type,const char *NameString)
{
  char String[STRLEN], buffer[STRLEN];
  int r;

  if (LoadString(AfxGetInstanceHandle(),Resource,String,1024)==0)
	r = AfxMessageBox("Could not find specified resource!  There is a fault in the file TinyCAD.EXE, please re-install it.",MB_ICONEXCLAMATION | MB_OK);
  else {
	// Play the sound associated with this message
	MessageBeep(Type & 0x70);
	sprintf(buffer,String,NameString);
	r = AfxMessageBox(buffer,Type);
  }

  return r;
}



////// The mouse movement operators //////

void CTinyCadView::OnMouseMove(UINT nFlags, CPoint p)
{

	Context theContext(this,theTransform);

	if (vRuler != NULL)
		vRuler->ShowPosition(p);
	if (hRuler != NULL)
		hRuler->ShowPosition(p);


	if (!m_pDocument)
	  return;

	CPoint snap_p=theTransform.DeScale(GetDocument()->m_snap,p);




	// If we are panning, then don't track this movement...
	if (m_panning)
	{
		// Pan the screen with the mouse
		int x = theTransform.GetOrigin().x;
		int y = theTransform.GetOrigin().y;
		CPoint old_mouse = MousePosition;
		SetScroll(x - snap_p.x + old_mouse.x, y - snap_p.y + old_mouse.y);

		MousePosition=theTransform.DeScale(GetDocument()->m_snap,p);
	}
	else
	{

		MousePosition=snap_p;
	
		// Now display the position
		CString pos = GetDocument()->GetOptions()->PointToUnit(snap_p);
		static_cast<CMainFrame*>(AfxGetMainWnd())->setPositionText( pos );

		// Send this to the editable device
		GetDocument()->GetEdit()->Move(snap_p);
	}
}

void CTinyCadView::OnLButtonDown(UINT nFlags, CPoint p)
{
  Context theContext(this,theTransform);

  CPoint snapped_p = theTransform.DeScale(GetDocument()->m_snap,p);
  CPoint no_snap_p = theTransform.DeScale(p);

  MousePosition=snapped_p;

  GetDocument()->GetEdit()->LButtonDown(snapped_p,no_snap_p);

  SetCapture();
  m_captured = TRUE;
}

void CTinyCadView::OnLButtonDblClk(UINT nFlags, CPoint p) 
{
  Context theContext(this,theTransform);

  CPoint snapped_p = theTransform.DeScale(GetDocument()->m_snap,p);
  CPoint no_snap_p = theTransform.DeScale(p);

  MousePosition=snapped_p;

  GetDocument()->GetEdit()->DblLButtonDown(snapped_p,no_snap_p);
	
  CView::OnLButtonDblClk(nFlags, p);
}

void CTinyCadView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_captured)
	{
		ReleaseCapture();

		Context theContext(this,theTransform);

		CPoint p=theTransform.DeScale(GetDocument()->m_snap,point);

		MousePosition=p;

		GetDocument()->GetEdit()->LButtonUp(p);
	}
	
	CView::OnLButtonUp(nFlags, point);
}



BOOL CTinyCadView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	// trackers should only be in client area
	if (nHitTest == HTCLIENT)
	{
		CPoint point;
		GetCursorPos(&point);
		pWnd->ScreenToClient(&point);
		CPoint no_snap_p = theTransform.DeScale(point);
		int cursor = GetDocument()->GetEdit()->SetCursor( no_snap_p );

		if (cursor != -1)
		{
			::SetCursor(m_mouse_pointers[cursor]);
			return TRUE;
		}
	}

	return CView::OnSetCursor(pWnd, nHitTest, message);
}





void CTinyCadView::OnRButtonDown(UINT nFlags, CPoint p)
{
  // Create a Context for this Click
  Context theContext(this,theTransform);

  CPoint snapped_p = theTransform.DeScale(GetDocument()->m_snap,p);
  CPoint no_snap_p = theTransform.DeScale(p);

  MousePosition=snapped_p;

  // Call the end function, if it returns false then delete
  // this object
  if (!GetDocument()->GetEdit()->RButtonDown(snapped_p,no_snap_p))
  {
	GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));
  }
}

void CTinyCadView::OnRButtonUp(UINT nFlags, CPoint p)
{
  Context theContext(this,theTransform);
  CPoint snapped_p = theTransform.DeScale(GetDocument()->m_snap,p);
  CPoint no_snap_p = theTransform.DeScale(p);

  MousePosition=snapped_p;

  GetDocument()->GetEdit()->RButtonUp(snapped_p, no_snap_p );
}


void CTinyCadView::OnMButtonDown( UINT nFlags, CPoint point )
{
	m_panning = TRUE;
	SetCapture();
}

void CTinyCadView::OnMButtonUp( UINT nFlags, CPoint point )
{
	m_panning = FALSE;
	ReleaseCapture();
}




// This function takes file names and shortens them (if necessary)
CString NameLength(const char *s,int MaxLen)
{
  CString in = s;
  int len = in.GetLength();
  int diff = MaxLen - len;

  // Is the string short enough as it is?
  if (diff>=0)
	return in;

  // Otherwise remove sufficient characters so it is
  if ((diff&1)!=0)
	diff++;

  return (in.Left((len+diff)/2-2) + "..." + in.Mid((len-diff)/2+1));
}


void CTinyCadView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	switch (lHint)
	{
	case DOC_UPDATE_SETCURSOR:
		{
			int cursor = GetDocument()->GetEdit()->SetCursor( MousePosition );
			if (cursor != -1)
			{
				::SetCursor(m_mouse_pointers[cursor]);
			}
		}
		break;
	case DOC_UPDATE_INVALIDATE:
		Invalidate();
		break;
	case DOC_UPDATE_INVALIDRECT:
	case DOC_UPDATE_INVALIDRECTERASE:
		doc_invalidrect* hint = static_cast<doc_invalidrect*>(pHint);

		Context dc(this, theTransform );
		dc.InvalidateRect( hint->r, lHint == DOC_UPDATE_INVALIDRECTERASE, hint->grow );
		break;
	}
	
}


void CTinyCadView::OnRulerVert() 
{
	GetDocument()->SelectObject(new CDrawRuler(GetDocument(),FALSE));
	
}

void CTinyCadView::OnUpdateRulerVert(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}

}

void CTinyCadView::OnRulerHoriz() 
{
	GetDocument()->SelectObject(new CDrawRuler(GetDocument(),TRUE));
}

void CTinyCadView::OnUpdateRulerHoriz(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}	
}


void CTinyCadView::OnUpdateEditedit(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateBusback(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateBusslash(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolarc(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolbus(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolbusname(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolcircle(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolconnect(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolget(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateTooljunc(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToollabel(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolpolygon(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolpower(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolsquare(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateTooltext(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateToolwire(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnUpdateViewcentre(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}


void CTinyCadView::OnUpdateEditdrag(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}	
}

void CTinyCadView::OnUpdateEditdup(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
	
}

void CTinyCadView::OnUpdateEditrotate(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}	
}

void CTinyCadView::OnUpdateEditmove(CCmdUI* pCmdUI) 
{
	CDrawingObject *q = GetDocument()->GetEdit();
	if (q)	
	{
		pCmdUI->SetCheck( q->getMenuID() == pCmdUI->m_nID );
	}
}

void CTinyCadView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	if (((CTinyCadApp*)AfxGetApp())->GetMdiMaximize() )
	{
		((CMDIChildWnd *)GetParentFrame())->MDIMaximize();
	}
	
}



////// The Snap to Grid menu //////

void CTinyCadView::OnSnaptogrid() 
{
	GetDocument()->SetSnapToGrid( !GetDocument()->GetSnapToGrid() );
}

void CTinyCadView::OnUpdateSnaptogrid(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetDocument()->GetSnapToGrid() ? 1 : 0 );	
}






////// The REPEAT menu //////

void CTinyCadView::OnRepeatNameUp()
{
	GetDocument()->SetNameDir( 1 );
}

void CTinyCadView::OnUpdateRepeatnameup(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetDocument()->GetNameDir() == 1 ? 1 : 0 );
	
}

void CTinyCadView::OnRepeatNameDown()
{
	GetDocument()->SetNameDir( -1 );
}

void CTinyCadView::OnUpdateRepeatnamedown(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetDocument()->GetNameDir() == -1 ? 1 : 0 );
}


void CTinyCadView::OnRepeatPinUp()
{
  GetDocument()->SetPinDir( 1 );
}

void CTinyCadView::OnUpdateRepeatpinup(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetDocument()->GetPinDir() == 1 ? 1 : 0 );
	
}


void CTinyCadView::OnRepeatPinDown()
{
	GetDocument()->SetPinDir( -1 );
}


void CTinyCadView::OnUpdateRepeatpindown(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetDocument()->GetPinDir() == -1 ? 1 : 0 );
}




void CTinyCadView::OnUpdateEditpaste(CCmdUI* pCmdUI) 
{
	BOOL r = IsClipboardAvailable() || ::IsClipboardFormatAvailable( CF_ENHMETAFILE );
	pCmdUI->Enable( r );	
}

void CTinyCadView::OnUpdateEditcut(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->IsSelected() );	
}

void CTinyCadView::OnUpdateEditcopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( TRUE );
}

void CTinyCadView::OnUpdateEditduplicate(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->IsSelected() );	
}

void CTinyCadView::OnUpdateEditCopyto(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->IsSelected() );	
}



BOOL CTinyCadView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	if (zDelta > 0)
	{
		OnViewZoomIn();
	}
	else
	{
		OnViewZoomOut();
	}
	
	return TRUE;
}

void CTinyCadView::OnLibraryRefreshsymbolsfromlibraries() 
{
	GetDocument()->ResetSymbols();
}

void CTinyCadView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	if (!bActivate)
	{
		// Get rid of any drawing tool at this moment
		GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));
	}
	
	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}



void CTinyCadView::OnViewOptions() 
{
	COptionsPropertySheet propSheet;

	propSheet.m_pDocument = GetDocument();
	propSheet.DoModal();

  if (hRuler != NULL)
  	hRuler->OnNewTransform(theTransform);
  if (vRuler != NULL)
  	vRuler->OnNewTransform(theTransform);

  Invalidate();
}

void CTinyCadView::OnEditDelete()
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, IDM_EDITDELITEM );

}

void CTinyCadView::OnContextMakehorizontal() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_MAKEHORIZONTAL );
	
}

void CTinyCadView::OnContextMakevertical() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_MAKEVERTICAL );
	
}


void CTinyCadView::OnContextArcin() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_ARCIN );
	
}

void CTinyCadView::OnContextArcout() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_ARCOUT );
}

void CTinyCadView::OnContextFreeline() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_FREELINE );
}

void CTinyCadView::OnContextAddhandle() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_ADDHANDLE );
}

void CTinyCadView::OnContextDeletehandle() 
{
	GetDocument()->GetEdit()->ContextMenu( MousePosition, ID_CONTEXT_DELETEHANDLE );
}

void CTinyCadView::OnContextCanceldrawing() 
{
	// switch back to the Edit tool
	GetDocument()->SelectObject( new CDrawEditItem(GetDocument()) );	
}

void CTinyCadView::OnContextFinishdrawing() 
{
	GetDocument()->GetEdit()->FinishDrawing(MousePosition);
}



void CTinyCadView::OnContextZorderBringtofront() 
{
	GetDocument()->BringToFront();
}

void CTinyCadView::OnContextZorderSendtoback() 
{
	GetDocument()->SendToBack();
}

void CTinyCadView::OnUpdateContextZorderBringtofront(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->IsSelected() );		
}

void CTinyCadView::OnUpdateContextZorderSendtoback(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->IsSelected() );		
}


void CTinyCadView::OnFileSaveasbitmap() 
{

	// Get the file in which to save the network
	char szFile[256];

	strcpy(szFile,GetDocument()->GetPathName());
	for (int i=0;szFile[i]!=0 && szFile[i]!='.';i++) ;
	strcpy(szFile+i, ".png");

	CDlgExportPNG dlg;
	dlg.m_Filename = szFile;

	if (dlg.DoModal() != IDOK)
		return;

	CClientDC dc( this );
	GetDocument()->SavePNG( dlg.m_Filename, dc, dlg.m_type != 0  );
}




void CTinyCadView::OnOptionsColours() 
{
	CDlgColours dlg;
	for (int i = 0; i < 7; i++)
	{
		dlg.m_colours[i] = GetDocument()->GetOptions()->GetColour( static_cast<user_colour>(i) );
	}

	int id = dlg.DoModal();
	if (id == IDOK)
	{
		for (int i = 0; i < 7; i++)
		{
			GetDocument()->GetOptions()->SetColour( static_cast<user_colour>(i), dlg.m_colours[i] );
		}
		RedrawWindow();
	}
	else if (id == IDC_RESET)
	{
		GetDocument()->GetOptions()->ResetColours();
		for (int i = 0; i < 7; i++)
		{
			user_colour u = static_cast<user_colour>(i);
			GetDocument()->GetOptions()->SetColour( u, GetDocument()->GetOptions()->GetColour(u) );
		}
		RedrawWindow();
	}
}
