/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <assert.h>
#include "TinyCadView.h"
#include "registry.h"
#include "TinyCad.h"
#include "LibraryDb.h"
#include "DlgLibraryBox.h"
#include "LibraryCollection.h"

//*************************************************************************
//*                                                                       *
//* The dialog for editing which libraries are in use                     *
//*                                                                       *
//*************************************************************************

BEGIN_MESSAGE_MAP( CDlgLibraryBox, CDialog )
	//{{AFX_MSG_MAP(CDlgLibraryBox)
	ON_COMMAND( LIBRARYBOX_ADD, OnAdd )
	ON_COMMAND( LIBRARYBOX_REMOVE, OnRemove )
	ON_COMMAND( LIBRARYBOX_EDIT, OnEdit )
	ON_LBN_DBLCLK(LIBRARYBOX_AVAILABLE, OnDblclkAvailable)
	ON_LBN_SELCHANGE(LIBRARYBOX_AVAILABLE, OnSelchangeAvailable)
	ON_BN_CLICKED(LIBRARYBOX_NEW, OnNew)
	ON_BN_CLICKED(LIBRARYBOX_UPGRADE, OnUpgrade)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//=========================================================================
//== ctor/dtor/initializing                                              ==
//=========================================================================

//-------------------------------------------------------------------------
CDlgLibraryBox::CDlgLibraryBox(CWnd* pParentWnd)
: CDialog( IDD_LIBRARY, pParentWnd )
{
	//{{AFX_DATA_INIT(CDlgLibraryBox)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}
//-------------------------------------------------------------------------
//-- Creating and filling listbox
BOOL CDlgLibraryBox::OnInitDialog()
{
	// Set up the list box with the current library names in it
	m_lstLibs = (CListBox*) GetDlgItem( LIBRARYBOX_AVAILABLE );

	RefreshComponents( true );

	return TRUE;
}
//-------------------------------------------------------------------------

//==========================================================================
//== accessor                                                             ==
//==========================================================================

//-------------------------------------------------------------------------
//-- Returns a reference on the current selected listbox item,
//-- otherwise NULL.
CLibraryStore* CDlgLibraryBox::GetSelectedLibrary() const
{
	CLibraryStore*	oReturn		= NULL;
	int				nSelIndex	= m_lstLibs->GetCurSel();
	CString			sLibName;

	// Get the item which has been selected
	if( nSelIndex != LB_ERR )
	{
		// Now calculate which library this is
		m_lstLibs->GetText( nSelIndex, sLibName );

		oReturn = CLibraryCollection::GetLibrary( sLibName );
	}

	return oReturn;
}
//-------------------------------------------------------------------------

//=========================================================================
//== mutator                                                             ==
//=========================================================================

//-------------------------------------------------------------------------
//-- Controls En/Disabling of some buttons of the dialog
//-- Parameter bHasToRefill forces redrawing of listbox
void CDlgLibraryBox::RefreshComponents( bool bHasToRefill )
{
	CButton*	btnRemove		= static_cast<CButton*>(GetDlgItem( LIBRARYBOX_REMOVE ));
	CButton*	btnEdit			= static_cast<CButton*>(GetDlgItem( LIBRARYBOX_EDIT ));
	CButton*	btnUpgrade		= static_cast<CButton*>(GetDlgItem( LIBRARYBOX_UPGRADE ));
	BOOL		bIsLibSelected	= m_lstLibs->GetCurSel() != LB_ERR;

	if( bHasToRefill )
	{
		CLibraryCollection::FillLibraryNames( m_lstLibs );

		// refilling unselects the listbox
		bIsLibSelected = false;
	}

	// activ if any listbox entry is selected
	btnRemove->EnableWindow( bIsLibSelected );
	btnEdit->EnableWindow( bIsLibSelected );
	btnUpgrade->EnableWindow( bIsLibSelected );
}
//-------------------------------------------------------------------------


//=========================================================================
//== message handling                                                    ==
//=========================================================================

//-------------------------------------------------------------------------
//-- Closes dialog and saves library pathes to registry
void CDlgLibraryBox::OnOK()
{
	CLibraryCollection::SaveToRegistry();
	EndDialog( IDOK );
}
//-------------------------------------------------------------------------
//-- Proofs vality of a library
//-- Adds it to collection and listbox
void CDlgLibraryBox::OnAdd()
{
	OPENFILENAME ofn;
  	char szFile[256], szFileTitle[256];

  	strcpy(szFile, "*.mdb;*.idx\0");

  	ofn.lStructSize = sizeof(OPENFILENAME);
  	ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  	ofn.lpstrFilter = "Library file (*.mdb, *idx)\0*.mdb;*.idx\0All files (*.*)\0*.*\0\0";
  	ofn.lpstrCustomFilter = NULL;
  	ofn.nMaxCustFilter = 0;
  	ofn.nFilterIndex = 1;
  	ofn.lpstrFile = szFile;
  	ofn.nMaxFile = sizeof(szFile);
  	ofn.lpstrFileTitle = szFileTitle;
  	ofn.nMaxFileTitle = sizeof(szFileTitle);
  	ofn.lpstrInitialDir = NULL;
  	ofn.lpstrTitle = NULL;
  	ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  	ofn.nFileOffset = 0;
  	ofn.nFileExtension = 0;
  	ofn.lpstrDefExt = ".mdb";

  	if( GetOpenFileName(&ofn) )
	{
  		// Remove the extension
  		for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.'; lp++) ;
  		szFile[lp]=0;

  		// Create the library
  		CLibraryStore* NewLib;

  		if (stricmp(szFile+lp+1,"idx") == 0)
  		{
			NewLib = new CLibraryFile;
  		}
  		else
  		{
			NewLib = new CLibraryDb;
  		}

  		NewLib->Attach( szFile );

		// Now add it to the library linked list
		if( ! CLibraryCollection::Contains( NewLib ) )
		{
			CLibraryCollection::Add( NewLib );
			RefreshComponents( true );

			// Finally update the symbol picker...
			static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();
		}
	}
}
//-------------------------------------------------------------------------
//-- Removes a library from collection and listbox
void CDlgLibraryBox::OnRemove()
{
	int				nSelIndex = m_lstLibs->GetCurSel();
	CString			sLibName;
	CLibraryStore*	pLib;

	// Get the item which has been selected
	if( nSelIndex != LB_ERR  )
	{
		// Now calculate which library this is
		m_lstLibs->GetText( nSelIndex, sLibName );

		pLib = CLibraryCollection::GetLibrary( sLibName );

		// listbox and collection should be synchron
		assert( pLib != NULL );

		if( static_cast<CTinyCadApp*>(AfxGetApp())->IsLibInUse( pLib ))
		{
			// We cannot delete whilst in use
			AfxMessageBox( IDS_NODELETE );
		}
		else
		{
			CLibraryCollection::Remove( pLib );
			RefreshComponents( true );

			// Finally update the symbol picker...
			static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();
		}
	}
}
//-------------------------------------------------------------------------
//-- Opens a thumbnail view of the selected library
void CDlgLibraryBox::OnEdit()
{
	CLibraryStore* pLib = GetSelectedLibrary();

	// Should be impossible to edit a non-selected library
	assert( pLib != NULL );

	static_cast<CTinyCadApp*>(AfxGetApp())->editLibrary( pLib );
	EndDialog( IDOK );
}
//-------------------------------------------------------------------------
//--
void CDlgLibraryBox::OnNew()
{
  	OPENFILENAME ofn;
  	char szFile[256], szFileTitle[256];

  	strcpy(szFile, "*.mdb\0");

  	ofn.lStructSize = sizeof(OPENFILENAME);
  	ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  	ofn.lpstrFilter = "Library database (*.mdb)\0*.mdb\0Old Library File (*.idx)\0*.idx\0All files (*.*)\0*.*\0\0";
  	ofn.lpstrCustomFilter = NULL;
  	ofn.nMaxCustFilter = 0;
  	ofn.nFilterIndex = 1;
  	ofn.lpstrFile = szFile;
  	ofn.nMaxFile = sizeof(szFile);
  	ofn.lpstrFileTitle = szFileTitle;
  	ofn.nMaxFileTitle = sizeof(szFileTitle);
  	ofn.lpstrInitialDir = NULL;
  	ofn.lpstrTitle = NULL;
  	ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  	ofn.nFileOffset = 0;
  	ofn.nFileExtension = 0;
  	ofn.lpstrDefExt = "mdb";

  	if( GetSaveFileName(&ofn) )
  	{
	  	// Remove the extension
	  	for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.'; lp++) ;
	  	szFile[lp]=0;

	  	// Create the library object
	  	CLibraryDb* NewLib = new CLibraryDb;

	  	// .. and create the file
	  	if( NewLib->Create(szFile) )
	  	{
	  		// Now add it to the library linked list
	  		CLibraryCollection::Add( NewLib );

	  		// .. and to the list box
			RefreshComponents( true );
	  	}
	  	else
	  	{
		  	delete NewLib;
		  	return;
	  	}
	}
}
//-------------------------------------------------------------------------
//-- Converts a library from the old format to
//-- the new Microsoft Access database format.
void CDlgLibraryBox::OnUpgrade()
{
	CLibraryStore* pOldLib = GetSelectedLibrary();

	if( pOldLib != NULL )
	{
		if( static_cast<CTinyCadApp*>(AfxGetApp())->IsLibInUse( pOldLib ) )
		{
			// We cannot upgrade whilst in use
			AfxMessageBox( IDS_NOUPGRADE );
		}
		else
		{
			CLibraryStore* pNewLib = new CLibraryDb;

			if( pOldLib->Upgrade( pNewLib ) )
			{
				// First remove the old library from this system
				CLibraryCollection::Remove( pOldLib );

				// Now add the new library
				CLibraryCollection::Add( pNewLib );
			}
			else
			{
				// Upgrade failed
				delete pNewLib;
			}

			// Finally update the symbol picker...
			static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();
		}
	}
}
//-------------------------------------------------------------------------
//-- single click on listbox refreshes it
void CDlgLibraryBox::OnSelchangeAvailable()
{
	RefreshComponents( false );
}
//-------------------------------------------------------------------------
//-- double click opens a thumbnail view of the selected library
void CDlgLibraryBox::OnDblclkAvailable()
{
	OnEdit();
}
//-------------------------------------------------------------------------

